#
# raccp.rb
#
#    Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#

require 'racc/parser'

$RACCPARSER_DEBUG = false

class Racc


  class RaccParser < Parser

    attr :start
    attr :classname


    def initialize( racc )
      racc.must Racc
      @racc   = racc
      @interf = racc.interf

      @scanner = RaccScanner.new
      @scanner.debug = @yydebug = $RACCPARSER_DEBUG
    end


    def parse( str, fname )
      fname.must String
      @scanner.reset( str )
      @scanner.filename = @filename = fname

      do_parse
    end


    private


    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      case val
      when Fixnum then v = val.id2name
      when String then v = val
      else
        v = val.inspect
      end
      raise ParseError, <<MES
  parse error:#{@filename}:#{@scanner.lineno}: unexpected token '#{v}'
MES
    end

##### racc version 0.10.3 generates ###

LR_reduce_table = [
0, 0, :racc_error,
6, 20, :_reduce_1,
0, 21, :_reduce_2,
2, 21, :_reduce_3,
3, 23, :_reduce_4,
1, 23, :_reduce_5,
2, 23, :_reduce_6,
2, 24, :_reduce_7,
3, 24, :_reduce_8,
3, 25, :_reduce_9,
3, 25, :_reduce_10,
1, 27, :_reduce_11,
2, 27, :_reduce_12,
2, 28, :_reduce_13,
2, 28, :_reduce_14,
2, 28, :_reduce_15,
1, 29, :_reduce_16,
2, 29, :_reduce_17,
1, 26, :_reduce_18,
1, 26, :_reduce_19,
2, 22, :_reduce_20,
3, 22, :_reduce_21,
3, 30, :_reduce_22,
3, 30, :_reduce_23,
0, 31, :_reduce_24,
2, 31, :_reduce_25,
1, 32, :_reduce_26,
1, 32, :_reduce_27,
1, 32, :_reduce_28,
2, 32, :_reduce_29 ]

LR_reduce_n = 30

LR_shift_n = 52

LR_action_table = [
     2,     2,    -1,   -30,     0,     3,    -1,   -30,     3,     4,
    -1,   -30,     0,     5,    -1,   -30,    -1,    -2,    -1,    52,
    10,     7,     6,     9,     4,    11,     7,    12,     9,    13,
    -1,   -30,    11,    15,    13,    16,    12,    17,    -1,   -30,
    -1,    -3,     3,    22,     8,    21,    -1,   -30,    -1,    -5,
     3,    25,    -1,   -30,     3,    26,    -1,   -30,    11,    15,
    13,    16,    12,    17,    -1,   -30,    -1,   -11,     3,    22,
     8,    21,    -1,   -30,     3,    22,     8,    21,    -1,   -30,
     3,    22,     8,    21,    -1,   -30,    11,    15,     9,    33,
    13,    16,    12,    17,    -1,   -30,     5,    35,     3,    22,
     8,    21,    -1,   -30,     8,    36,    -1,   -30,    -1,   -19,
    -1,   -18,    16,    37,    14,    38,    -1,   -30,     5,    40,
     3,    25,    -1,   -30,    15,    41,    -1,   -30,    -1,    -6,
    10,    42,    11,    15,    13,    16,    12,    17,    -1,   -30,
    -1,   -16,     3,    22,     8,    21,    -1,   -13,     3,    22,
     8,    21,    -1,   -15,     3,    22,     8,    21,    -1,   -14,
    -1,   -12,    -1,   -10,     8,    44,    -1,   -30,    -1,    -4,
    -1,    -7,    -1,   -24,    -1,   -20,    16,    37,    14,    46,
    -1,   -30,    -1,    -1,    -1,   -24,    -1,    -9,    -1,   -17,
    -1,    -8,    18,    48,     3,    51,     8,    50,    17,    49,
    -1,   -23,    -1,   -21,    18,    48,     3,    51,     8,    50,
    17,    49,    -1,   -22,     3,    22,     8,    21,    -1,   -30,
    -1,   -28,    -1,   -27,    -1,   -26,    -1,   -25,    -1,   -29 ]

LR_action_table_ptr = [
     0,     4,     8,    12,    16,    18,    20,    32,    40,    42,
    48,    50,    54,    58,    66,    68,    74,    80,    86,    96,
   104,   108,   110,   112,   118,   124,   128,   130,   140,   142,
   148,   154,   160,   162,   164,   168,   170,   172,   174,   176,
   182,   184,   186,   188,   190,   192,   202,   204,   214,   220,
   222,   224,   226,   228 ]


LR_goto_table = [
    20,     1,    21,     6,    23,     8,    25,    10,    28,    14,
    27,    18,    24,    19,    26,    20,    30,    23,    22,    24,
    28,    14,    27,    27,    26,    28,    29,    29,    26,    28,
    29,    30,    26,    28,    29,    31,    28,    32,    26,    34,
    30,    39,    28,    32,    26,    43,    26,    43,    26,    43,
    31,    45,    31,    47,    32,    52,    32,    52,    26,    53,
    -1,    -1 ]

LR_goto_table_ptr = [
     0,    -1,    -1,    -1,     2,    -1,     4,     8,    -1,    12,
    -1,    16,    -1,    20,    -1,    24,    28,    32,    36,    38,
    -1,    -1,    -1,    -1,    40,    -1,    -1,    42,    -1,    44,
    46,    48,    -1,    -1,    -1,    -1,    -1,    50,    -1,    -1,
    -1,    52,    -1,    -1,    -1,    54,    -1,    56,    58,    -1,
    -1,    -1,    -1,    -1 ]


LR_token_table = {
 false => 0,
 Object.new => 1,
 :CLASS => 2,
 :TOKEN => 3,
 :RULE => 4,
 :XEND => 5,
 :XTOKEN => 6,
 :START => 7,
 :STRING => 8,
 :PRECHIGH => 9,
 :PRECLOW => 10,
 :LEFT => 11,
 :RIGHT => 12,
 :NONASSOC => 13,
 ";" => 14,
 ":" => 15,
 "|" => 16,
 :ACTION => 17,
 "=" => 18 }

DEBUG_PARSER = false

##### racc system variables end #####

 def _reduce_0( val, _values, result )

  result
 end

 def _reduce_1( val, _values, result )
      @interf.end_register_rule
      @classname = if Integer === val[1] then val[1].id2name else val[1] end
  result
 end

 def _reduce_2( val, _values, result )

  result
 end

 def _reduce_3( val, _values, result )

  result
 end

 def _reduce_4( val, _values, result )
      @interf.end_register_conv
  result
 end

 def _reduce_5( val, _values, result )

  result
 end

 def _reduce_6( val, _values, result )
      if @start then
        raise ParseError, "start rule defined twice"
      end
      @start = @interf.get_token( val[1] )
  result
 end

 def _reduce_7( val, _values, result )
      @interf.register_conv( val[0], val[1] )
  result
 end

 def _reduce_8( val, _values, result )
      @interf.register_conv( val[1], val[2] )
  result
 end

 def _reduce_9( val, _values, result )
      @interf.end_register_prec( true )
  result
 end

 def _reduce_10( val, _values, result )
      @interf.end_register_prec( false )
  result
 end

 def _reduce_11( val, _values, result )

  result
 end

 def _reduce_12( val, _values, result )

  result
 end

 def _reduce_13( val, _values, result )
      @interf.register_prec( :Left, val[1] )
  result
 end

 def _reduce_14( val, _values, result )
      @interf.register_prec( :Right, val[1] )
  result
 end

 def _reduce_15( val, _values, result )
      @interf.register_prec( :Nonassoc, val[1] )
  result
 end

 def _reduce_16( val, _values, result )
      result = val
  result
 end

 def _reduce_17( val, _values, result )
      result.push val[1]
  result
 end

 def _reduce_18( val, _values, result )
      result = @interf.get_token( val[0] )
  result
 end

 def _reduce_19( val, _values, result )
      result = @interf.get_token( eval '"' + val[0] + '"' )
  result
 end

 def _reduce_20( val, _values, result )

  result
 end

 def _reduce_21( val, _values, result )

  result
 end

 def _reduce_22( val, _values, result )
      @interf.register_rule @interf.get_token(val[0]), val[2]
  result
 end

 def _reduce_23( val, _values, result )
      @interf.register_rule nil, val[2]
  result
 end

 def _reduce_24( val, _values, result )
      result = []
  result
 end

 def _reduce_25( val, _values, result )
      result.push val[1]
  result
 end

 def _reduce_26( val, _values, result )
      result = @interf.get_token( val[0] )
  result
 end

 def _reduce_27( val, _values, result )
      result = @interf.get_token( eval '"' + val[0] + '"' )
  result
 end

 def _reduce_28( val, _values, result )

  result
 end

 def _reduce_29( val, _values, result )
      @interf.register_tmpprec val[1]
  result
 end

  end

end
